%%
% Replication code for "Understanding HANK: Insights from a PRANK"
% Authors: Sushant Acharya and Keshav Dogra
% February 4, 2020
% Code for model with persistent income case: Plots Figures 4 and 5 in
% Appendix

%% Housekeeping
clear;
clc;
close all;

%% Parameters

% Parameters for HANK
param.gam = 3;
param.r = 0.04;
param.mu = param.r/(1+param.r);
param.kap = 0.1; % slope of phillips curve
param.thet = 1; % note, we can always find some dsigdy that generates this
param.sigss = 0.02;0.1^2;
param.btil = 1-param.mu;
param.babylam = 0.97; % persistence of skill shock
param.lam = param.gam*(param.r/(1+param.r-param.babylam))*param.sigss;
param.truebeta = (1/(1+param.r))*exp(- param.sigss*(param.gam*param.r/(1+param.r-param.babylam))^2);

% Parameters for RANK ( we keep beta the same between HANK and RANK)
parRA = param;
parRA.lam=0;
parRA.thet=1;
parRA.btil = param.truebeta;
parRA.mu = 1- parRA.btil;
parRA.r  = parRA.btil^(-1) -1;

extreme = 5;
baseline = -1;
dsdy = [extreme,0,baseline,-extreme];
thets = 1 - (1/2)*param.gam*(((1-param.btil)^2)/(1-param.btil*param.lam))*dsdy;

%% Figure 4. Forward guidance experiments
% compute equilibrium under a promise to reduce i_t by eps k periods into
% the future, for different thetas

eps = 1;
K = 6;
nt = length(thets);
y_fg = zeros(nt,K+1);
y_ra = zeros(1,K+1);

% sequence of shocks
ehist = zeros(3,K);
ehist(1,K) = -eps;

% HANK with different cyclicalities of risk
for it=1:nt
    param.thet = thets(it);
    [y_fg(it,1:K+1), ~, ~, ~] = solvepeg_noniid(param,ehist);
end

% RANK
[y_ra(1,1:K+1), ~, ~] = solvepeg(parRA,ehist);

figure()
plot(0:K,y_ra(1,1:K+1),'k:','LineWidth',1.5)
hold on
plot(0:K,y_fg(4,1:K+1),'k','LineWidth',1.5)
plot(0:K,y_fg(3,1:K+1),'r--','MarkerFaceColor','r','LineWidth',1.5)
plot(0:K,y_fg(2,1:K+1),'b','LineWidth',1.5)
plot(0:K,y_fg(1,1:K+1),'Color',[0 0.6 1],'LineWidth',2)
hl=legend('RANK','HANK $\frac{d\sigma^2}{dy} = -5$','HANK $\frac{d\sigma^2}{dy} = -1$','HANK $\frac{d\sigma^2}{dy} = 0$','HANK $\frac{d\sigma^2}{dy} = 5$','Interpreter','latex');
set(hl ,'Interpreter','latex')
xlabel('time')
xticks(0:5)
axis([0 K-1 1/param.gam 0.6])
set(gcf, 'PaperPosition', [0 0 6 4]);
set(gcf, 'PaperSize', [6 4]);
saveas(gca,'y_fgp_noniid_6.pdf','pdf')
saveas(gca,'y_fgp_noniid_6.eps','epsc')


% Plot Figure 4b) varying duration of FG

K=10; % max duration of FG
y_fg = zeros(nt,K+1);
y_ra = zeros(1,K+1);

% sequence of shocks
ehist = zeros(3,K);
ehist(1,K) = -eps;

% HANK with different cyclicalities of risk
for it=1:nt
    param.thet = thets(it);
    [y_fg(it,1:K+1), ~, ~, ~] = solvepeg_noniid(param,ehist);
end
gf_y = fliplr(y_fg(:,1:end-1));

%RANK
[y_ra(1,1:K+1), ~, ~] = solvepeg(parRA,ehist);
ar_y = fliplr(y_ra(:,1:end-1));

figure()
plot(1:K,ar_y(1,1:K),'k:','LineWidth',1.5)
hold on
plot(1:K,gf_y(4,1:K),'k','LineWidth',1.5)
plot(1:K,gf_y(3,1:K),'r--','MarkerFaceColor','r','LineWidth',1.5)
plot(1:K,gf_y(2,1:K),'b','LineWidth',1.5)
plot(1:K,gf_y(1,1:K),'Color',[0 0.6 1],'LineWidth',2)
hl=legend('RANK','HANK $\frac{d\sigma^2}{dy} = -5$','HANK $\frac{d\sigma^2}{dy} = -1$','HANK $\frac{d\sigma^2}{dy} = 0$','HANK $\frac{d\sigma^2}{dy} = 5$','Interpreter','latex');
set(hl ,'Interpreter','latex','Location','NorthWest')
xlabel('k')
xticks(1:K)
axis([1 K 1/param.gam 1.2])
set(gcf, 'PaperPosition', [0 0 6 4]);
set(gcf, 'PaperSize', [6 4]);
saveas(gca,'y_fgp_dur_noniid_6.pdf','pdf')
saveas(gca,'y_fgp_dur_noniid_6.eps','epsc')

%% Figure 5. Govt spending multipliers at the ZLB

% Plot Figure 5a)
K = 5; % length of trap

y_g = zeros(nt,K+1);
y_rag = zeros(1,K+1);

% sequence of shocks
ehist = zeros(3,K);
ehist(3,K) = 1; % g_t - g_t+1

% HANK
for it=1:nt
    param.thet = thets(it);
    [y_g(it,1:K+1), ~,~, ~] = solvepeg_noniid(param,ehist);
end

%RANK
[y_rag(1,1:K+1), ~, ~] = solvepeg(parRA,ehist);

figure()
plot(0:K-1,y_rag(1,1:K),'k:','LineWidth',1.5)
hold on
plot(0:K-1,y_g(4,1:K),'k','LineWidth',1.5)
plot(0:K-1,y_g(3,1:K),'r--','MarkerFaceColor','r','LineWidth',1.5)
plot(0:K-1,y_g(2,1:K),'b','LineWidth',1.5)
plot(0:K-1,y_g(1,1:K),'Color',[0 0.6 1],'LineWidth',2)
hl=legend('RANK','HANK $\frac{d\sigma^2}{dy} = -5$','HANK $\frac{d\sigma^2}{dy} = -1$','HANK $\frac{d\sigma^2}{dy} = 0$','HANK $\frac{d\sigma^2}{dy} = 5$','Interpreter','latex');
set(hl ,'Interpreter','latex')
xlabel('time')
xticks(0:5)
axis([0 K-1 1 1.5])
set(gcf, 'PaperPosition', [0 0 6 4]);
set(gcf, 'PaperSize', [6 4]);
saveas(gca,'y_gov_noniid_6.pdf','pdf')
saveas(gca,'y_gov_noniid_6.eps','epsc')

% Figure 5 b) varying duration of trap
K=10;
y_g = zeros(nt,K+1);
y_rag = zeros(1,K+1);

% sequence of shocks
ehist = zeros(3,K);
ehist(3,K) = 1; % g_t - g_t+1


% HANK
for it=1:nt
    param.thet = thets(it);
    [y_g(it,1:K+1), ~, ~, ~] = solvepeg_noniid(param,ehist);
end
g_y   = fliplr(y_g(:,1:end-1));

% RANK
[y_rag(1,1:K+1), ~, ~] = solvepeg(parRA,ehist);
gar_y = fliplr(y_rag(:,1:end-1));


figure()
plot(1:K,gar_y(1,1:K),'k:','LineWidth',1.5)
hold on
plot(1:K,g_y(4,1:K),'k','LineWidth',1.5)
plot(1:K,g_y(3,1:K),'r--','MarkerFaceColor','r','LineWidth',1.5)
plot(1:K,g_y(2,1:K),'b','LineWidth',1.5)
plot(1:K,g_y(1,1:K),'Color',[0 0.6 1],'LineWidth',2)
hl=legend('RANK','HANK $\frac{d\sigma^2}{dy} = -5$','HANK $\frac{d\sigma^2}{dy} = -1$','HANK $\frac{d\sigma^2}{dy} = 0$','HANK $\frac{d\sigma^2}{dy} = 5$','Interpreter','latex');
set(hl ,'Interpreter','latex','Location','NorthWest')
xlabel('duration')
xticks(1:K)
axis([1 K 1 2.2])
set(gcf, 'PaperPosition', [0 0 6 4]);
set(gcf, 'PaperSize', [6 4]);
saveas(gca,'y_gov_dur_noniid_6.pdf','pdf')
saveas(gca,'y_gov_dur_noniid_6.eps','epsc')