%%
% Replication code for "Understanding HANK: Insights from a PRANK"
% Authors: Sushant Acharya and Keshav Dogra
% February 4, 2020

%% Housekeeping
clear;
clc;
close all;

%% Set parameters

param.gam   = 3;                  % coefficient of prudence
param.r     = 0.04;                 % real interest rate
param.mu    = param.r/(1+param.r); % steady state mpc
param.kap   = 0.1;                % slope of phillips curve
param.thet  = 1;                 % cyclicality of risk (initializing)
param.sigss = 0.25;             % this is the variance of income; std=0.5
param.truebeta = (1/(1+param.r))*exp(- param.sigss*(param.gam*param.mu)^2); %the beta consistent with given risk and interest rate in steady state
param.btil = 1-param.mu;        % inverse real interest rate in steady state
param.lam = param.gam*(param.mu*param.mu)*param.sigss;  % steady state consumption risk scaled by gamma

% set of parameters for RANK ( we keep beta the same across HANK and RANK )
parRA = param;
parRA.lam=0; % no risk
parRA.thet=1;% no risk, no cyclicality
parRA.btil = param.truebeta;  
parRA.mu = 1- parRA.btil;
parRA.r  = parRA.btil^(-1) -1;

% create grid of values for cyclicality of risk
extreme = 5;
baseline = -1;
dsdy = [extreme,0,baseline,-extreme];
thets = 1 - (1/2)*param.gam*param.mu*param.mu*dsdy;

%% Figure 1. Forward guidance experiments
% compute equilibrium under a promise to reduce i_t by eps=1 k periods into
% the future, for different thetas

% plot Figure 1 (a)
eps = 1;
K = 5; % length of FG
nt = length(thets);

y_fg = zeros(nt,K+1);
y_ra = zeros(1,K+1);

% sequence of shocks
ehist = zeros(3,K); 
ehist(1,K) = -eps;

% HANK with different cyclialities of risk
for it=1:nt
    param.thet = thets(it);
    [y_fg(it,1:K+1), ~, ~] = solvepeg(param,ehist); % solve for path of output for different values of cyclicality of risk
end
    
% RANK
[y_ra(1,1:K+1), ~, ~] = solvepeg(parRA,ehist);

figure()
plot(0:K,y_ra(1,1:K+1),'k:','LineWidth',1.5)
hold on
plot(0:K,y_fg(4,1:K+1),'k','LineWidth',1.5)
plot(0:K,y_fg(3,1:K+1),'r--','MarkerFaceColor','r','LineWidth',1.5)
plot(0:K,y_fg(2,1:K+1),'b','LineWidth',1.5)
plot(0:K,y_fg(1,1:K+1),'Color',[0 0.6 1],'LineWidth',2)
hl=legend('RANK','HANK $\frac{d\sigma^2}{dy} = -5$','HANK $\frac{d\sigma^2}{dy} = -1$','HANK $\frac{d\sigma^2}{dy} = 0$','HANK $\frac{d\sigma^2}{dy} = 5$','Interpreter','latex');
set(hl ,'Interpreter','latex')
xlabel('time')
xticks(0:5)
axis([0 K-1 1/param.gam 0.47])
set(gcf, 'PaperPosition', [0 0 6 4]); 
set(gcf, 'PaperSize', [6 4]); 
saveas(gca,'y_fgp_6.pdf','pdf') 
saveas(gca,'y_fgp_6.eps','epsc') 


% plot Figure 1 (b)
K=10; % horizon of FG
y_fg = zeros(nt,K+1);
y_ra = zeros(1,K+1);

% shock sequence
ehist = zeros(3,K);
ehist(1,K) = -eps;

% HANK with different cyclicalities of risk
for it=1:nt
    param.thet = thets(it);
    [y_fg(it,1:K+1), ~, ~] = solvepeg(param,ehist);
end
gf_y = fliplr(y_fg(:,1:end-1));

% RANK
[y_ra(1,1:K+1), ~, ~] = solvepeg(parRA,ehist);
ar_y = fliplr(y_ra(:,1:end-1));


figure()
plot(1:K,ar_y(1,1:K),'k:','LineWidth',1.5)
hold on
plot(1:K,gf_y(4,1:K),'k','LineWidth',1.5)
plot(1:K,gf_y(3,1:K),'r--','MarkerFaceColor','r','LineWidth',1.5)
plot(1:K,gf_y(2,1:K),'b','LineWidth',1.5)
plot(1:K,gf_y(1,1:K),'Color',[0 0.6 1],'LineWidth',1.5)
hl=legend('RANK','HANK $\frac{d\sigma^2}{dy} = -5$','HANK $\frac{d\sigma^2}{dy} = -1$','HANK $\frac{d\sigma^2}{dy} = 0$','HANK $\frac{d\sigma^2}{dy} = 5$','Interpreter','latex');
set(hl ,'Interpreter','latex')
set(hl,'Location','NorthWest')
xlabel('k')
xticks(1:K)
axis([1 K 1/3 0.95])
set(gcf, 'PaperPosition', [0 0 6 4]); 
set(gcf, 'PaperSize', [6 4]); 
saveas(gca,'y_fgp_dur_6.pdf','pdf') 
saveas(gca,'y_fgp_dur_6.eps','epsc') 

%% Figure 2. Govt spending multipliers at the ZLB

% Plot Figure 2a
K = 5; % length of trap
nt = length(thets);

y_g = zeros(nt,K+1);
y_rag = zeros(1,K+1);

% sequence of shocks
ehist = zeros(3,K);
ehist(3,K) = 1; % g_t - g_t+1

% HANK with different cyclicalities of risk
for it=1:nt
    param.thet = thets(it);
    [y_g(it,1:K+1), ~, ~] = solvepeg(param,ehist);
end

% RANK
[y_rag(1,1:K+1), ~, ~] = solvepeg(parRA,ehist);

figure()
plot(0:K,y_rag(1,1:K+1),'k:','LineWidth',1.5)
hold on
plot(0:K,y_g(4,1:K+1),'k','LineWidth',1.5)
plot(0:K,y_g(3,1:K+1),'r--','MarkerFaceColor','r','LineWidth',1.5)
plot(0:K,y_g(2,1:K+1),'b','LineWidth',1.5)
plot(0:K,y_g(1,1:K+1),'Color',[0 0.6 1],'LineWidth',1.5)
hl=legend('RANK','HANK $\frac{d\sigma^2}{dy} = -5$','HANK $\frac{d\sigma^2}{dy} = -1$','HANK $\frac{d\sigma^2}{dy} = 0$','HANK $\frac{d\sigma^2}{dy} = 5$','Interpreter','latex');
set(hl ,'Interpreter','latex')
axis([0 K-1 1 1.38])
xlabel('time')
xticks(0:K-1)
set(gcf, 'PaperPosition', [0 0 6 4]); 
set(gcf, 'PaperSize', [6 4]); 
saveas(gca,'y_gov_6.pdf','pdf') 
saveas(gca,'y_gov_6.eps','epsc') 
hold off

% Plot Figure 2b
K=10; % max horizon of trap
y_g = zeros(nt,K+1);
y_rag = zeros(1,K+1);

% sequence of shocks
ehist = zeros(3,K);
ehist(3,K) = 1; % g_t - g_t+1

% HANK with different cyclicalities of risk
for it=1:nt
    param.thet = thets(it);
    [y_g(it,1:K+1), ~, ~] = solvepeg(param,ehist);
end
g_y   = fliplr(y_g(:,1:end-1));

% RANK
[y_rag(1,1:K+1), mu_rag(1,1:K+1), pi_rag(1,1:K+1)] = solvepeg(parRA,ehist);
gar_y = fliplr(y_rag(:,1:end-1));


figure()
plot(1:K,gar_y(1,1:K),'k:','LineWidth',1.5)
hold on
plot(1:K,g_y(4,1:K),'k','LineWidth',1.5)
plot(1:K,g_y(3,1:K),'r--','MarkerFaceColor','r','LineWidth',1.5)
plot(1:K,g_y(2,1:K),'b','LineWidth',1.5)
plot(1:K,g_y(1,1:K),'Color',[0 0.6 1],'LineWidth',1.5)
hl=legend('RANK','HANK $\frac{d\sigma^2}{dy} = -5$','HANK $\frac{d\sigma^2}{dy} = -1$','HANK $\frac{d\sigma^2}{dy} = 0$','HANK $\frac{d\sigma^2}{dy} = 5$','Interpreter','latex','Location','NorthWest');
set(hl ,'Interpreter','latex')
xlabel('time')
xticks(1:K)
xlabel('duration')
axis([1 K 1 2.5])
set(gcf, 'PaperPosition', [0 0 6 4]); 
set(gcf, 'PaperSize', [6 4]); 
saveas(gca,'y_gov_dur_6.pdf','pdf') 
saveas(gca,'y_gov_dur_6.eps','epsc') 